/*  cpu.c
 *
 *  Copyright (c) 2007-2008 Angelo Arrifano <miknix@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "cpu.h"

#define PROC_STAT "/proc/stat"
#define PROCMAXLNLEN 256 /* should make it */

cpuLoadData *data = NULL;
int nrCpus = 0;

int cpuLoadMon_init()
{
	FILE *fStat = NULL;
	char  cpuStr[PROCMAXLNLEN];
	int   cpuNr = -1;

	/* Check if previously initalized */
	if(data != NULL) return(-2);

	/* Open proc stat file */
	if(!(fStat = fopen(PROC_STAT, "r"))) return(-1);

	/* Read each cpu line at time */
	do
	{
		if(!fgets(cpuStr, PROCMAXLNLEN, fStat)) return(cpuNr);
		cpuNr++;
	}
	while(strncmp(cpuStr, "cpu", 3) == 0);

	/* Alloc storage for cpu data stuff */
	data = (cpuLoadData *) calloc(cpuNr, sizeof(cpuLoadData));
	if(data == NULL) return(-3);

	fclose(fStat);
	return(nrCpus=cpuNr);
}

void cpuLoadMon_free()
{
	/* free cpu data */
	free(data);
	data = NULL;

	/* Reset values */
	nrCpus = 0;
}


cpuLoadData *cpuLoadMon_read(){
	FILE *fStat = NULL;
	char cpuStr[PROCMAXLNLEN];
	unsigned long user, nice, system, idle;
	unsigned long used, total;
	int cpuNr = 0;
	
	
	/* Check if callable */
	if((data == NULL) || (nrCpus == 0)) return(NULL);

	/* Open proc stat file */
	if(!(fStat = fopen(PROC_STAT, "r"))) return(NULL);

	/* Read each cpu line at time */
	do
	{
		if(!fgets(cpuStr, PROCMAXLNLEN, fStat)) return(data);
		sscanf(cpuStr, "%*s %ld %ld %ld %ld", &user, &nice, &system, &idle);
		
		used = user + nice + system;
		total = used + idle;
		if((total - data[cpuNr].pTotal) != 0)
		{
			data[cpuNr].value = (float)(used - data[cpuNr].pUsed)
				/ (float)(total - data[cpuNr].pTotal);
		}
		else
		{
			data[cpuNr].value = 0;
		}
		data[cpuNr].pUsed = used;
		data[cpuNr].pTotal = total;
		cpuNr++;
	}
	while((cpuNr < nrCpus) && (strncmp(cpuStr, "cpu", 3) == 0));

	fclose(fStat);
	return(data);
}
