/*  cpuload-dialogs.c
 *
 *  Copyright (c) 2007-2008 Angelo Arrifano <miknix@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include <gtk/gtk.h>

#include <libxfcegui4/libxfcegui4.h>
#include <libxfce4panel/xfce-panel-plugin.h>

#include "cpuload.h"
#include "cpuload-dialogs.h"

#define INTERVAL_MIN 50
#define INTERVAL_MAX 1000
#define INTERVAL_STEP 50

/* the website url */
#define PLUGIN_WEBSITE "http://goodies.xfce.org/projects/panel-plugins/xfce4-cpuload-plugin"



static void cpuLoad_configure_response(
	GtkWidget    *dialog,
	gint          response,
	cpuLoadPlugin *cpuLoad)
{
	gboolean result;

	if(response == GTK_RESPONSE_HELP){
		/* show help */
		result = g_spawn_command_line_async("exo-open --launch WebBrowser " PLUGIN_WEBSITE, NULL);

		if(G_UNLIKELY(result == FALSE))
			g_warning(_("Unable to open the following url: %s"), PLUGIN_WEBSITE);
	}
	else
	{
		/* remove the dialog data from the plugin */
		g_object_set_data(G_OBJECT (cpuLoad->plugin), "dialog", NULL);

		/* save the plugin */
		cpuLoad_save(cpuLoad->plugin, cpuLoad);

		/* destroy the properties dialog */
		gtk_widget_destroy(dialog);
	}
	xfce_panel_plugin_unblock_menu(cpuLoad->plugin);
}

void cpuLoad_configure_cpu_toggled(
	GtkToggleButton *togglebutton,
	cpuLoadPlugin   *cpuLoad)
{
	cpuLoad->showCpu = gtk_toggle_button_get_active(togglebutton);
	if(cpuLoad->showCpu)
		gtk_widget_show(cpuLoad->pBar[0]);
	else
		gtk_widget_hide(cpuLoad->pBar[0]);
}

void cpuLoad_configure_cpuColor_set(
	GtkColorButton *widget,
	cpuLoadPlugin  *cpuLoad)
{
	gtk_color_button_get_color(
		GTK_COLOR_BUTTON(widget),
		&cpuLoad->cpuColor);
	cpuLoad_pBar_color_update(cpuLoad);
}

void cpuLoad_configure_coreColor_set(
	GtkColorButton *widget,
	cpuLoadPlugin  *cpuLoad)
{
	gtk_color_button_get_color(
		GTK_COLOR_BUTTON(widget),
		&cpuLoad->coreColor);
	cpuLoad_pBar_color_update(cpuLoad);
}

void cpuLoad_configure(
	XfcePanelPlugin *plugin,
	cpuLoadPlugin    *cpuLoad)
{
	GtkWidget *dialog;
	GtkWidget *wid;
	GtkWidget *box;

	/* block the plugin menu */
	xfce_panel_plugin_block_menu(plugin);

	/* create the dialog */
	dialog = xfce_titled_dialog_new_with_buttons(_("CPU Load"),
		GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(plugin))),
		GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
		GTK_STOCK_HELP,
		GTK_RESPONSE_HELP,
		GTK_STOCK_CLOSE,
		GTK_RESPONSE_OK,
		NULL);

	/* center dialog on the screen */
	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);

	/* set dialog icon */
	gtk_window_set_icon_name(GTK_WINDOW (dialog), "xfce4-settings");

	/* link the dialog to the plugin, so we can destroy it when the plugin
	 * is closed, but the dialog is still open */
	g_object_set_data(G_OBJECT(plugin), "dialog", dialog);

	/* cpu check button */
	wid = gtk_check_button_new_with_label(_("Show CPU"));
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), wid, FALSE, FALSE, 0);
	gtk_widget_set_sensitive(wid, (cpuLoad->nrCores==1?FALSE:TRUE));
	gtk_toggle_button_set_active(
		GTK_TOGGLE_BUTTON(wid),
		cpuLoad->showCpu);
	g_signal_connect(
		G_OBJECT(wid),
		"toggled",
		G_CALLBACK(cpuLoad_configure_cpu_toggled),
		cpuLoad);
	gtk_widget_show(wid);

	/* cpu bar color */
	box = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), box, TRUE, TRUE, 4);
	gtk_widget_show(box);

	wid = gtk_label_new(_("CPU Color"));
	gtk_box_pack_start(GTK_BOX(box),	wid, TRUE, TRUE, 0);
	gtk_widget_show(wid);

	wid = gtk_color_button_new_with_color(&cpuLoad->cpuColor);
	gtk_color_button_set_use_alpha(GTK_COLOR_BUTTON(wid), FALSE);
	gtk_box_pack_start(GTK_BOX(box),	wid, FALSE, FALSE, 2);
	g_signal_connect(
		G_OBJECT(wid),
		"color-set",
		G_CALLBACK(cpuLoad_configure_cpuColor_set),
		cpuLoad);
	gtk_widget_show(wid);

	/* cpu core bar color */
	box = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), box, TRUE, TRUE, 4);
	gtk_widget_show(box);

	wid = gtk_label_new(_("CPU Core Color"));
	gtk_box_pack_start(GTK_BOX(box),	wid, TRUE, TRUE, 0);
	gtk_widget_show(wid);

	wid = gtk_color_button_new_with_color(&cpuLoad->coreColor);
	gtk_color_button_set_use_alpha(GTK_COLOR_BUTTON(wid), FALSE);
	gtk_box_pack_start(GTK_BOX(box),	wid, FALSE, FALSE, 2);
	g_signal_connect(
		G_OBJECT(wid),
		"color-set",
		G_CALLBACK(cpuLoad_configure_coreColor_set),
		cpuLoad);
	gtk_widget_show(wid);

	/* connect the reponse signal to the dialog */
	g_signal_connect (
		G_OBJECT(dialog),
		"response",
		G_CALLBACK(cpuLoad_configure_response),
		cpuLoad);

	/* show the entire dialog */
	gtk_widget_show (dialog);
}



void cpuLoad_about (XfcePanelPlugin *plugin)
{
	GtkWidget *dlg;

	dlg = gtk_message_dialog_new(
		GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(plugin))),
		GTK_DIALOG_MODAL,
		GTK_MESSAGE_INFO,
		GTK_BUTTONS_OK,
		"XFCE CPU Load Plugin v" VERSION "\n\nDeveloped by Angelo Arrifano <miknix@gmail.com>\nSee README for more info.");
	
	/* center dialog on the screen */
	gtk_window_set_position(GTK_WINDOW(dlg), GTK_WIN_POS_CENTER);

	gtk_dialog_run(GTK_DIALOG(dlg));
	gtk_widget_destroy(dlg);
}
