/*  cpuload.c
 *
 *  Copyright (c) 2007-2008 Angelo Arrifano <miknix@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include <gtk/gtk.h>
#include <libxfce4util/libxfce4util.h>
#include <libxfce4panel/xfce-panel-plugin.h>
#include <libxfce4panel/xfce-hvbox.h>

#include "cpu.h"
#include "cpuload.h"
#include "cpuload-dialogs.h"

/* default settings */
#define SHOW_CPU TRUE
#define CPU_COLOR "#FF0000"
#define CORE_COLOR "#7600FF"

#define UPDATE_INTERVAL 250
#define BORDER 8

/* prototypes */
static void
cpuLoad_construct(XfcePanelPlugin *plugin);

/* register the plugin */
XFCE_PANEL_PLUGIN_REGISTER_EXTERNAL(cpuLoad_construct);


gboolean cpuLoad_update(cpuLoadPlugin *cpuLoad)
{
	gint i;
	cpuLoadData *data;
	
	data = cpuLoadMon_read();

	for(i=(cpuLoad->showCpu?0:1); i<cpuLoad->nrCores; i++){
		gtk_progress_bar_set_fraction(
			GTK_PROGRESS_BAR(cpuLoad->pBar[i]),
			(gdouble)data[i].value);
	}
	return(TRUE);
}


void cpuLoad_pBar_color_update(cpuLoadPlugin *cpuLoad)
{
	GtkRcStyle                *rc;
	gint                       i;

	/* change progress bar color */
	for(i=0; i<cpuLoad->nrCores; i++)
	{
		rc = gtk_widget_get_modifier_style(GTK_WIDGET(cpuLoad->pBar[i]));
		if(!rc) rc = gtk_rc_style_new();
		rc->color_flags[GTK_STATE_PRELIGHT] |= GTK_RC_BG;
		if(i == 0)
			rc->bg[GTK_STATE_PRELIGHT] = cpuLoad->cpuColor;
		else
			rc->bg[GTK_STATE_PRELIGHT] = cpuLoad->coreColor;
		gtk_widget_modify_style(GTK_WIDGET(cpuLoad->pBar[i]), rc);
	}
}


void cpuLoad_save(
	XfcePanelPlugin *plugin,
	cpuLoadPlugin    *cpuLoad)
{
	XfceRc *rc;
	gchar  *file;
	gchar  value[8];


	/* get the config file location */
	file = xfce_panel_plugin_save_location(plugin, TRUE);

	if(G_UNLIKELY(file == NULL))
	{
		DBG("Failed to open config file");
		return;
	}

	/* open the config file, read/write */
	rc = xfce_rc_simple_open(file, FALSE);
	g_free(file);

	if(G_LIKELY(rc != NULL))
	{
		/* save the settings */
		xfce_rc_write_bool_entry(rc, "ShowCPU", cpuLoad->showCpu);

		/* CPU Color */
		g_snprintf(value, 8, "#%02X%02X%02X",
			(guint)cpuLoad->cpuColor.red >> 8,
			(guint)cpuLoad->cpuColor.green >> 8,
			(guint)cpuLoad->cpuColor.blue >> 8);
		xfce_rc_write_entry(rc, "CPUColor", value);

		/* GPU Color */
		g_snprintf(value, 8, "#%02X%02X%02X",
			(guint)cpuLoad->coreColor.red >> 8,
			(guint)cpuLoad->coreColor.green >> 8,
			(guint)cpuLoad->coreColor.blue >> 8);
		xfce_rc_write_entry(rc, "CoreColor", value);

      /* close the rc file */
      xfce_rc_close(rc);
	}
}



void cpuLoad_read(cpuLoadPlugin *cpuLoad)
{
	XfceRc      *rc;
	gchar       *file;
	const gchar *value;

	/* get the plugin config file location */
	file = xfce_panel_plugin_save_location(cpuLoad->plugin, TRUE);

	if(G_LIKELY(file != NULL))
	{
		/* open the config file, readonly */
		rc = xfce_rc_simple_open(file, TRUE);
		g_free(file);

		if(G_LIKELY(rc != NULL))
		{
			/* read the settings */
			cpuLoad->showCpu = xfce_rc_read_bool_entry(rc, "ShowCPU", SHOW_CPU);

			value = xfce_rc_read_entry(rc, "CPUColor", CPU_COLOR);
			gdk_color_parse(value, &cpuLoad->cpuColor);

			value = xfce_rc_read_entry(rc, "CoreColor", CORE_COLOR);
			gdk_color_parse(value, &cpuLoad->coreColor);

			/* cleanup */
			xfce_rc_close(rc);

			/* leave the function, everything went well */
			return;
		}
	}

	/* something went wrong, apply default values */
	DBG ("Applying default settings");

	cpuLoad->showCpu = SHOW_CPU;
	gdk_color_parse(CPU_COLOR, &cpuLoad->cpuColor);
	gdk_color_parse(CORE_COLOR, &cpuLoad->coreColor);
}

static cpuLoadPlugin *cpuLoad_new(XfcePanelPlugin   *plugin)
{
	cpuLoadPlugin             *cpuLoad;
	GtkOrientation             orientation;
	GtkProgressBarOrientation  barOrientation;
	gint                       i;

	/* allocate memory for the plugin structure */
	//cpuLoad = panel_slice_new0(cpuLoadPlugin);
	cpuLoad = g_new(cpuLoadPlugin, 1);

	/* pointer to plugin */
	cpuLoad->plugin = plugin;

	/* read the user settings */
	cpuLoad_read(cpuLoad);

	/* get the current orientation */
	orientation = xfce_panel_plugin_get_orientation(plugin);
	if(orientation == GTK_ORIENTATION_HORIZONTAL)
		barOrientation = GTK_PROGRESS_BOTTOM_TO_TOP;
	else
		barOrientation = GTK_PROGRESS_LEFT_TO_RIGHT;

	/* create container */
	cpuLoad->ebox = gtk_event_box_new();
	gtk_widget_show(cpuLoad->ebox);

	cpuLoad->hvbox = xfce_hvbox_new(orientation, FALSE, 0);
	gtk_container_add(GTK_CONTAINER(cpuLoad->ebox), cpuLoad->hvbox);
	gtk_container_set_border_width(
		GTK_CONTAINER(cpuLoad->hvbox),
		BORDER / 2);
		gtk_widget_show(cpuLoad->hvbox);

	/* Initialize monitor */
	if((cpuLoad->nrCores = cpuLoadMon_init()) < 1)
		DBG("Cannot init cpuLoad monitor!\n");

	/* If not SMP a system, must show the cpu */
	if(cpuLoad->nrCores == 1)
		cpuLoad->showCpu = TRUE;

	/* Alloc progress bars */
	cpuLoad->pBar =
		(GtkWidget **) malloc(sizeof(GtkWidget *) * cpuLoad->nrCores);

	/* Create progress bars */
	for(i=0; i<cpuLoad->nrCores; i++)
	{
		cpuLoad->pBar[i] = GTK_WIDGET(gtk_progress_bar_new());
		
		gtk_progress_bar_set_orientation(
			GTK_PROGRESS_BAR(cpuLoad->pBar[i]),
			barOrientation);
		
		gtk_box_pack_start(
			GTK_BOX(cpuLoad->hvbox),
			cpuLoad->pBar[i],
			FALSE,
			FALSE,
			1);

		/* show only cpu if option is set */	
		if((i > 0) || (cpuLoad->showCpu == TRUE))
			gtk_widget_show(cpuLoad->pBar[i]);
	}

	cpuLoad_pBar_color_update(cpuLoad);

	return(cpuLoad);
}



static void cpuLoad_free (
	XfcePanelPlugin *plugin,	
	cpuLoadPlugin    *cpuLoad)
{
	GtkWidget *dialog;
	gint i;

	/* free cpuload monitor */
	cpuLoadMon_free();

	/* check if the dialog is still open. if so, destroy it */
	dialog = g_object_get_data(G_OBJECT(plugin), "dialog");
	if(G_UNLIKELY(dialog != NULL))
		gtk_widget_destroy(dialog);

	/* destroy the panel widgets */
	for(i=0; i<cpuLoad->nrCores; i++)
		gtk_widget_destroy(cpuLoad->pBar[i]);
	gtk_widget_destroy(cpuLoad->hvbox);
	gtk_widget_destroy(cpuLoad->ebox);

	free(cpuLoad->pBar);

	/* free the plugin structure */
	panel_slice_free (cpuLoadPlugin, cpuLoad);
}

static void cpuLoad_orientation_changed(
	XfcePanelPlugin *plugin,
	GtkOrientation   orientation,
	cpuLoadPlugin   *cpuLoad)
{
	GtkProgressBarOrientation barOrientation;
	gpointer                  p_pBar[cpuLoad->nrCores];
	gint                      i;

	gtk_widget_hide(cpuLoad->ebox);

	/* get the current orientation */
	orientation = xfce_panel_plugin_get_orientation(plugin);
	if(orientation == GTK_ORIENTATION_HORIZONTAL)
		barOrientation = GTK_PROGRESS_BOTTOM_TO_TOP;
	else
		barOrientation = GTK_PROGRESS_LEFT_TO_RIGHT;

	/* Remove progress bars from hvbox */
	for(i=0; i<cpuLoad->nrCores; i++)
	{
		/* reference progress bars to keep them alive */
		p_pBar[i] = g_object_ref(cpuLoad->pBar[i]);
		gtk_container_remove(
			GTK_CONTAINER(cpuLoad->hvbox),
			GTK_WIDGET(cpuLoad->pBar[i]));
	}
	
	/* Change hvbox orientation */
	xfce_hvbox_set_orientation(XFCE_HVBOX(cpuLoad->hvbox), orientation);

	/* Pack progress bars again into hvbox */
	for(i=0; i<cpuLoad->nrCores; i++)
	{
		gtk_progress_bar_set_orientation(
			GTK_PROGRESS_BAR(cpuLoad->pBar[i]),
			barOrientation);	
		gtk_box_pack_start(
			GTK_BOX(cpuLoad->hvbox),
			cpuLoad->pBar[i],
			FALSE,
			FALSE,
			1);
		/* We dont need anymore this reference */
		g_object_unref(p_pBar[i]);
	}

	gtk_widget_show(cpuLoad->ebox);
}



static gboolean cpuLoad_size_changed(
	XfcePanelPlugin *plugin,
	gint             size,
	cpuLoadPlugin   *cpuLoad)
{
	gint           i;

	/* set the widget size */
	if(xfce_panel_plugin_get_orientation(plugin) == GTK_ORIENTATION_HORIZONTAL)
	{
		gtk_widget_set_size_request(GTK_WIDGET(plugin), -1, size);
		for(i=0; i<cpuLoad->nrCores; i++)
			gtk_widget_set_size_request(
				GTK_WIDGET(cpuLoad->pBar[i]),
				BORDER,
				size);
	}
	else
	{
  		gtk_widget_set_size_request(GTK_WIDGET(plugin), size, -1);
		for(i=0; i<cpuLoad->nrCores; i++)
			gtk_widget_set_size_request(
				GTK_WIDGET(cpuLoad->pBar[i]),
				size,
				BORDER);
	}

	/* we handled the orientation */
	return TRUE;
}



static void cpuLoad_construct(XfcePanelPlugin *plugin)
{
	cpuLoadPlugin *cpuLoad;

	/* setup transation domain */
	xfce_textdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");

	/* create the plugin */
	cpuLoad = cpuLoad_new(plugin);

	/* add the ebox to the panel */
	gtk_container_add(GTK_CONTAINER(plugin), cpuLoad->ebox);

	/* show the panel's right-click menu on this ebox */
	xfce_panel_plugin_add_action_widget(plugin, cpuLoad->ebox);

	/* connect plugin signals */
	g_signal_connect(G_OBJECT(plugin), "free-data",
		G_CALLBACK (cpuLoad_free), cpuLoad);

	g_signal_connect(G_OBJECT(plugin), "save",
		G_CALLBACK (cpuLoad_save), cpuLoad);

	g_signal_connect(G_OBJECT(plugin), "size-changed",
		G_CALLBACK(cpuLoad_size_changed), cpuLoad);

	g_signal_connect(G_OBJECT(plugin), "orientation-changed",
		G_CALLBACK(cpuLoad_orientation_changed), cpuLoad);

	/* show the configure menu item and connect signal */
	xfce_panel_plugin_menu_show_configure(plugin);
	g_signal_connect(G_OBJECT(plugin), "configure-plugin",
		G_CALLBACK(cpuLoad_configure), cpuLoad);

	/* show the about menu item and connect signal */
	xfce_panel_plugin_menu_show_about(plugin);
	g_signal_connect(G_OBJECT(plugin), "about",
		G_CALLBACK(cpuLoad_about), cpuLoad);

	/* Start monitor */
	cpuLoad->timeoutID = g_timeout_add(
		UPDATE_INTERVAL,
		(GSourceFunc)cpuLoad_update,
		cpuLoad);
}
